==> sdk-3.0.0-sources.jar: com/kontakt/sdk/android/ble/manager/internal/SpacesResolver.java <==
package com.kontakt.sdk.android.ble.manager.internal;

import com.kontakt.sdk.android.ble.configuration.scan.ScanContext;
import com.kontakt.sdk.android.ble.device.BeaconRegion;
import com.kontakt.sdk.android.ble.device.EddystoneNamespace;
import com.kontakt.sdk.android.cloud.IKontaktCloud;
import com.kontakt.sdk.android.cloud.response.CloudCallback;
import com.kontakt.sdk.android.cloud.response.CloudError;
import com.kontakt.sdk.android.cloud.response.CloudHeaders;
import com.kontakt.sdk.android.cloud.response.paginated.Namespaces;
import com.kontakt.sdk.android.cloud.response.paginated.Proximities;
import com.kontakt.sdk.android.common.log.Logger;
import com.kontakt.sdk.android.common.model.Namespace;
import com.kontakt.sdk.android.common.model.ProximityId;
import com.kontakt.sdk.android.common.profile.IBeaconRegion;
import com.kontakt.sdk.android.common.profile.IEddystoneNamespace;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static com.kontakt.sdk.android.common.util.SDKPreconditions.checkNotNull;

/**
 * Class is responsible for resolving shuffled iBeacon regions or Eddystone namespaces using Kontakt Cloud.
 */
public abstract class SpacesResolver<Space> {

  public interface ResolvingStatusListener {
    void onSuccess();

    void onError(String message);
  }

  protected final IKontaktCloud cloud;
  protected final Map<String, Space> toResolve = new HashMap<>();
  protected final Collection<Space> resolved = new ArrayList<>();
  protected final Map<String, Space> cache = new HashMap<>();
  protected ResolvingStatusListener resolvingStatusListener;
  protected int maxResults = 50;
  private boolean isFinishedSuccessfully;

  public SpacesResolver(IKontaktCloud cloud) {
    this.cloud = checkNotNull(cloud);
  }

  public void resolve(ScanContext scanContext, ResolvingStatusListener onSpacesResolvedListener) {
    this.resolvingStatusListener = checkNotNull(onSpacesResolvedListener);
    isFinishedSuccessfully = false;
    clear();
    divideSpaces(getSpacesFromScanContext(scanContext));
    if (toResolve.isEmpty()) {
      invokeSuccessCallback();
      Logger.d("No shuffled spaces needs resolving.");
      return;
    }
    maxResults = toResolve.size() + 50;
    fetchSpacesFromCloud();
  }

  public void clear() {
    toResolve.clear();
    resolved.clear();
  }

  public void clearCache() {
    cache.clear();
  }

  public void resetFinishedStatus() {
    isFinishedSuccessfully = false;
  }

  public boolean isFinishedSuccessfully() {
    return isFinishedSuccessfully;
  }

  public Collection<Space> getResolvedSpaces() {
    return resolved;
  }

  protected abstract Collection<Space> getSpacesFromScanContext(ScanContext scanContext);

  protected abstract void divideSpaces(Collection<Space> spaces);

  protected abstract void fetchSpacesFromCloud();

  protected void invokeSuccessCallback() {
    isFinishedSuccessfully = true;
    resolvingStatusListener.onSuccess();
  }

  protected void invokeErrorCallback(String message) {
    resolvingStatusListener.onError(message);
  }

  public static class RegionsResolver extends SpacesResolver<IBeaconRegion> {

    public RegionsResolver(IKontaktCloud cloud) {
      super(cloud);
    }

    @Override
    protected Collection<IBeaconRegion> getSpacesFromScanContext(ScanContext scanContext) {
      return scanContext.getIBeaconScanContext().getSpaces();
    }

    @Override
    protected void divideSpaces(Collection<IBeaconRegion> iBeaconRegions) {
      for (IBeaconRegion region : iBeaconRegions) {
        if (region.isShuffled()) {
          if (cache.containsKey(region.getProximity().toString())) {
            resolved.add(cache.get(region.getProximity().toString()));
            continue;
          }
          toResolve.put(region.getProximity().toString(), region);
        } else {
          resolved.add(region);
        }
      }
    }

    @Override
    protected void fetchSpacesFromCloud() {
      cloud.proximities().fetch().maxResult(maxResults).execute(new CloudCallback<Proximities>() {
        @Override
        public void onSuccess(final Proximities proximities, CloudHeaders headers) {
          resolveRegions(proximities.getContent());
          invokeSuccessCallback();
          Logger.d("Successfully fetched spaces from Kontakt Cloud");
        }

        @Override
        public void onError(CloudError error) {
          invokeErrorCallback(error.getMessage());
        }
      });
    }

    private void resolveRegions(List<ProximityId> proximityUUIDs) {
      for (ProximityId proximityId : proximityUUIDs) {
        IBeaconRegion region = toResolve.get(proximityId.getProximityUUID().toString());
        if (region != null && proximityId.isShuffled() && proximityId.getProximityUUID().equals(region.getProximity())) {
          IBeaconRegion shuffledRegion = createNewRegion(region, proximityId.getSecureProximityUUID().toString());
          resolved.add(shuffledRegion);
          cache.put(region.getProximity().toString(), shuffledRegion);
        }
      }
    }

    private IBeaconRegion createNewRegion(IBeaconRegion region, String secureId) {
      return new BeaconRegion.Builder().setMajor(region.getMajor())
          .setMinor(region.getMinor())
          .setShuffled(region.isShuffled())
          .setIdentifier(region.getIdentifier())
          .setProximity(UUID.fromString(secureId))
          .build();
    }
  }

  public static class NamespacesResolver extends SpacesResolver<IEddystoneNamespace> {

    public NamespacesResolver(IKontaktCloud cloud) {
      super(cloud);
    }

    @Override
    protected Collection<IEddystoneNamespace> getSpacesFromScanContext(ScanContext scanContext) {
      return scanContext.getEddystoneScanContext().getSpaces();
    }

    @Override
    protected void divideSpaces(Collection<IEddystoneNamespace> namespaces) {
      for (IEddystoneNamespace namespace : namespaces) {
        if (namespace.isShuffled()) {
          if (cache.containsKey(namespace.getNamespace())) {
            resolved.add(cache.get(namespace.getNamespace()));
            continue;
          }
          toResolve.put(namespace.getNamespace(), namespace);
        } else {
          resolved.add(namespace);
        }
      }
    }

    @Override
    protected void fetchSpacesFromCloud() {
      cloud.namespaces().fetch().maxResult(maxResults).execute(new CloudCallback<Namespaces>() {
        @Override
        public void onSuccess(Namespaces namespaces, CloudHeaders headers) {
          resolveNamespaces(namespaces.getContent());
          invokeSuccessCallback();
          Logger.d("Successfully fetched namespaces from Kontakt Cloud");
        }

        @Override
        public void onError(CloudError error) {
          invokeErrorCallback(error.getMessage());
        }
      });
    }

    private void resolveNamespaces(List<Namespace> namespaces) {
      for (Namespace eddystoneNamespace : namespaces) {
        IEddystoneNamespace namespace = toResolve.get(eddystoneNamespace.getNamespaceId());
        if (namespace != null && eddystoneNamespace.isShuffled() && eddystoneNamespace.getNamespaceId().equals(namespace.getNamespace())) {
          IEddystoneNamespace shuffledNamespace = createNewNamespace(namespace, eddystoneNamespace.getSecureNamespaceId());
          resolved.add(shuffledNamespace);
          cache.put(namespace.getNamespace(), shuffledNamespace);
        }
      }
    }

    private IEddystoneNamespace createNewNamespace(IEddystoneNamespace oldSpace, String secureId) {
      return EddystoneNamespace.create(oldSpace.getName(), secureId, oldSpace.isShuffled());
    }
  }
}

==> sdk-3.1.0-sources.jar: com/kontakt/sdk/android/ble/manager/internal/SpacesResolver.java <==
package com.kontakt.sdk.android.ble.manager.internal;

import com.kontakt.sdk.android.ble.configuration.scan.ScanContext;
import com.kontakt.sdk.android.ble.device.BeaconRegion;
import com.kontakt.sdk.android.ble.device.EddystoneNamespace;
import com.kontakt.sdk.android.cloud.IKontaktCloud;
import com.kontakt.sdk.android.cloud.response.CloudCallback;
import com.kontakt.sdk.android.cloud.response.CloudError;
import com.kontakt.sdk.android.cloud.response.CloudHeaders;
import com.kontakt.sdk.android.cloud.response.paginated.Namespaces;
import com.kontakt.sdk.android.cloud.response.paginated.Proximities;
import com.kontakt.sdk.android.common.log.Logger;
import com.kontakt.sdk.android.common.model.Namespace;
import com.kontakt.sdk.android.common.model.ProximityId;
import com.kontakt.sdk.android.common.profile.IBeaconRegion;
import com.kontakt.sdk.android.common.profile.IEddystoneNamespace;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static com.kontakt.sdk.android.common.util.SDKPreconditions.checkNotNull;

/**
 * Class is responsible for resolving shuffled iBeacon regions or Eddystone namespaces using Kontakt Cloud.
 */
public abstract class SpacesResolver<Space> {

  public interface ResolvingStatusListener {
    void onSuccess();

    void onError(String message);
  }

  protected final IKontaktCloud cloud;
  protected final Map<String, Space> toResolve = new HashMap<>();
  protected final Collection<Space> resolved = new ArrayList<>();
  protected final Map<String, Space> cache = new HashMap<>();
  protected ResolvingStatusListener resolvingStatusListener;
  protected int maxResults = 50;
  private boolean isFinishedSuccessfully;

  public SpacesResolver(IKontaktCloud cloud) {
    this.cloud = checkNotNull(cloud);
  }

  public void resolve(ScanContext scanContext, ResolvingStatusListener onSpacesResolvedListener) {
    this.resolvingStatusListener = checkNotNull(onSpacesResolvedListener);
    isFinishedSuccessfully = false;
    clear();
    divideSpaces(getSpacesFromScanContext(scanContext));
    if (toResolve.isEmpty()) {
      invokeSuccessCallback();
      Logger.d("No shuffled spaces needs resolving.");
      return;
    }
    maxResults = toResolve.size() + 50;
    fetchSpacesFromCloud();
  }

  public void clear() {
    toResolve.clear();
    resolved.clear();
  }

  public void clearCache() {
    cache.clear();
  }

  public void resetFinishedStatus() {
    isFinishedSuccessfully = false;
  }

  public boolean isFinishedSuccessfully() {
    return isFinishedSuccessfully;
  }

  public Collection<Space> getResolvedSpaces() {
    return resolved;
  }

  protected abstract Collection<Space> getSpacesFromScanContext(ScanContext scanContext);

  protected abstract void divideSpaces(Collection<Space> spaces);

  protected abstract void fetchSpacesFromCloud();

  protected void invokeSuccessCallback() {
    isFinishedSuccessfully = true;
    resolvingStatusListener.onSuccess();
  }

  protected void invokeErrorCallback(String message) {
    resolvingStatusListener.onError(message);
  }

  public static class RegionsResolver extends SpacesResolver<IBeaconRegion> {

    public RegionsResolver(IKontaktCloud cloud) {
      super(cloud);
    }

    @Override
    protected Collection<IBeaconRegion> getSpacesFromScanContext(ScanContext scanContext) {
      return scanContext.getIBeaconRegions();
    }

    @Override
    protected void divideSpaces(Collection<IBeaconRegion> iBeaconRegions) {
      for (IBeaconRegion region : iBeaconRegions) {
        if (region.isShuffled()) {
          if (cache.containsKey(region.getProximity().toString())) {
            resolved.add(cache.get(region.getProximity().toString()));
            continue;
          }
          toResolve.put(region.getProximity().toString(), region);
        } else {
          resolved.add(region);
        }
      }
    }

    @Override
    protected void fetchSpacesFromCloud() {
      cloud.proximities().fetch().maxResult(maxResults).execute(new CloudCallback<Proximities>() {
        @Override
        public void onSuccess(final Proximities proximities, CloudHeaders headers) {
          resolveRegions(proximities.getContent());
          invokeSuccessCallback();
          Logger.d("Successfully fetched spaces from Kontakt Cloud");
        }

        @Override
        public void onError(CloudError error) {
          invokeErrorCallback(error.getMessage());
        }
      });
    }

    private void resolveRegions(List<ProximityId> proximityUUIDs) {
      for (ProximityId proximityId : proximityUUIDs) {
        IBeaconRegion region = toResolve.get(proximityId.getProximityUUID().toString());
        if (region != null && proximityId.isShuffled() && proximityId.getProximityUUID().equals(region.getProximity())) {
          IBeaconRegion shuffledRegion = createNewRegion(region, proximityId.getSecureProximityUUID().toString());
          resolved.add(shuffledRegion);
          cache.put(region.getProximity().toString(), shuffledRegion);
        }
      }
    }

    private IBeaconRegion createNewRegion(IBeaconRegion region, String secureId) {
      return new BeaconRegion.Builder().setMajor(region.getMajor())
          .setMinor(region.getMinor())
          .setShuffled(region.isShuffled())
          .setIdentifier(region.getIdentifier())
          .setProximity(UUID.fromString(secureId))
          .build();
    }
  }

  public static class NamespacesResolver extends SpacesResolver<IEddystoneNamespace> {

    public NamespacesResolver(IKontaktCloud cloud) {
      super(cloud);
    }

    @Override
    protected Collection<IEddystoneNamespace> getSpacesFromScanContext(ScanContext scanContext) {
      return scanContext.getEddystoneNamespaces();
    }

    @Override
    protected void divideSpaces(Collection<IEddystoneNamespace> namespaces) {
      for (IEddystoneNamespace namespace : namespaces) {
        if (namespace.isShuffled()) {
          if (cache.containsKey(namespace.getNamespace())) {
            resolved.add(cache.get(namespace.getNamespace()));
            continue;
          }
          toResolve.put(namespace.getNamespace(), namespace);
        } else {
          resolved.add(namespace);
        }
      }
    }

    @Override
    protected void fetchSpacesFromCloud() {
      cloud.namespaces().fetch().maxResult(maxResults).execute(new CloudCallback<Namespaces>() {
        @Override
        public void onSuccess(Namespaces namespaces, CloudHeaders headers) {
          resolveNamespaces(namespaces.getContent());
          invokeSuccessCallback();
          Logger.d("Successfully fetched namespaces from Kontakt Cloud");
        }

        @Override
        public void onError(CloudError error) {
          invokeErrorCallback(error.getMessage());
        }
      });
    }

    private void resolveNamespaces(List<Namespace> namespaces) {
      for (Namespace eddystoneNamespace : namespaces) {
        IEddystoneNamespace namespace = toResolve.get(eddystoneNamespace.getNamespaceId());
        if (namespace != null && eddystoneNamespace.isShuffled() && eddystoneNamespace.getNamespaceId().equals(namespace.getNamespace())) {
          IEddystoneNamespace shuffledNamespace = createNewNamespace(namespace, eddystoneNamespace.getSecureNamespaceId());
          resolved.add(shuffledNamespace);
          cache.put(namespace.getNamespace(), shuffledNamespace);
        }
      }
    }

    private IEddystoneNamespace createNewNamespace(IEddystoneNamespace oldSpace, String secureId) {
      return EddystoneNamespace.create(oldSpace.getName(), secureId, oldSpace.isShuffled());
    }
  }
}
